
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#include "Aabb.hh"
#include "Plane.hh"
#include "Sphere.hh"


namespace Mf {


Plane::Halfspace Plane::intersects(const Aabb& aabb) const
{
	Vector3 corners[8];
	int nPositive = 8;

	aabb.getCorners(corners);

	for (int i = 0; i < 8; ++i)
	{
		if (intersects(corners[i]) == NEGATIVE)
		{
			--nPositive;
		}
	}

	if (nPositive == 0)      return NEGATIVE;
	else if (nPositive == 8) return POSITIVE;
	else                     return INTERSECT;
}

Plane::Halfspace Plane::intersects(const Sphere& sphere) const
{
	Scalar distance = getDistanceToPoint(sphere.point);

	if (distance < -sphere.radius)     return NEGATIVE;
	else if (distance < sphere.radius) return INTERSECT;
	else                               return POSITIVE;
}


} // namespace Mf

/** vim: set ts=4 sw=4 tw=80: *************************************************/

