
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_SOUND_HH_
#define _MOOF_SOUND_HH_

/**
 * @file Sound.hh
 * Load and play sounds, current supports ogg vorbis.
 */

#include <boost/shared_ptr.hpp>

#include <Moof/Exception.hh>
#include <Moof/Math.hh>
#include <Moof/Resource.hh>


namespace Mf {


class Sound;
typedef boost::shared_ptr<Sound> SoundP;


class Sound : public Resource
{
	class Impl;
	boost::shared_ptr<Impl> impl_;

public:

	static SoundP alloc(const std::string& name)
	{
		return SoundP(new Sound(name));
	}

	explicit Sound(const std::string& name);


	void play();
	void stream();

	// TODO - i don't like how there are two different methods that essentially
	// do the same thing; the API should be exactly the same for both types of
	// sounds; need a different way to distinguish how to handle the sound

	void stop();
	void pause();
	void resume();
	void toggle();

	void setSample(const std::string& name);
	void enqueue(const std::string& name);

	bool isPlaying() const;

	void setPosition(Vector3 position);
	void setVelocity(Vector3 velocity);
	void setGain(Scalar gain);
	void setPitch(Scalar pitch);
	void setLooping(bool looping);

	static std::string getPath(const std::string& name);


	struct Exception : public Mf::Exception
	{
		enum
		{
			BAD_AUDIO_FORMAT	= 1024
		};

		explicit Exception(unsigned error) :
			Mf::Exception(error) {}

		void raise()
		{
			throw *this;
		}

		const char* what() const throw()
		{
			switch (code)
			{
				case BAD_AUDIO_FORMAT:
					return "unknown audio format";
			}
			return Mf::Exception::what();
		}
	};
};


} // namespace Mf

#endif // _MOOF_SOUND_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

